RECORD_DFT_ATTR = {
    'name',
    'created',
    'msecs',
    'relativeCreated',
    'asctime',
    'levelname',
    'levelno',
    'pathname',
    'filename',
    'module',
    'lineno',
    'funcName',
    'thread',
    'threadName',
    'processName',
    'process',
    'exc_info',
    'exc_text',
    'stack_info',
    'msg',
    'args',
    'message'
}



VERSION = (1, 2, 0, 'alpha1')
if isinstance(VERSION[-1], str):
    # Support for alpha version: 0.1.0-alpha1
    __version__ = "-".join([".".join(map(str, VERSION[:-1])), VERSION[-1]])
else:
    __version__ = ".".join(map(str, VERSION))



import json
import logging
import sys
import unittest
from collections import OrderedDict

from logging_utilities.filters import ConstAttribute
from logging_utilities.formatters.json_formatter import JsonFormatter

# From python3.7, dict is ordered
if sys.version_info >= (3, 7):
    dictionary = dict
else:
    dictionary = OrderedDict


class BasicJsonFormatterTest(unittest.TestCase):
    maxDiff = None

    @classmethod
    def _configure_logger(cls, logger, fmt=None, add_always_extra=False):
        logger.setLevel(logging.DEBUG)

        for handler in logger.handlers:
            formatter = JsonFormatter(fmt, add_always_extra=add_always_extra)
            handler.setFormatter(formatter)

    @classmethod
    def _configure_logger_with_const_attribute(cls, logger, remove_empty=False):
        logger.setLevel(logging.DEBUG)

        for handler in logger.handlers:
            const_attribute = ConstAttribute(application="test_application", empty_attr='')
            handler.addFilter(const_attribute)
            formatter = JsonFormatter(
                dictionary([
                    ('level', 'levelname'),
                    ('app', 'application'),
                    ('empty_attr', 'empty_attr'),
                    ('message', 'message'),
                ]),
                remove_empty=remove_empty,
                filter_attributes=['application', 'empty_attr']
            )
            handler.setFormatter(formatter)

    def test_simple_log(self):
        with self.assertLogs('test_formatter', level=logging.DEBUG) as ctx:
            logger = logging.getLogger('test_formatter')
            self._configure_logger(logger)
            logger.info('Simple message')
            logger.info('Composed message: %s', 'this is a composed message')
            logger.info('Composed message %s', 'with extra', extra={'extra1': 23})
        message1 = json.loads(ctx.output[0], object_pairs_hook=dictionary)
        message2 = json.loads(ctx.output[1], object_pairs_hook=dictionary)
        message3 = json.loads(ctx.output[2], object_pairs_hook=dictionary)
        self.assertDictEqual(
            message1,
            dictionary([
                ("levelname", "INFO"),
                ("name", "test_formatter"),
                ("message", "Simple message"),
            ])
        )
        self.assertDictEqual(
            message2,
            dictionary([
                ("levelname", "INFO"),
                ("name", "test_formatter"),
                ("message", "Composed message: this is a composed message"),
            ])
        )
        self.assertDictEqual(
            message3,
            dictionary([
                ("levelname", "INFO"),
                ("name", "test_formatter"),
                ("message", "Composed message with extra"),
            ])
        )

    def test_embedded_object(self):
        with self.assertLogs('test_formatter', level=logging.DEBUG) as ctx:
            logger = logging.getLogger('test_formatter')
            self._configure_logger(
                logger,
                fmt=dictionary([
                    ('levelname', 'levelname'),
                    ('msg', 'message'),
                    (
                        'system',
                        dictionary([
                            ('context', dictionary([
                                ('module', 'module'),
                                ('file', 'filename'),
                            ]))
                        ])
                    ),
                ])
            )
            logger.info('Simple message')
            logger.info('Composed message: %s', 'this is a composed message')
            logger.info('Composed message %s', 'with extra', extra={'extra1': 23})
        message1 = json.loads(ctx.output[0], object_pairs_hook=dictionary)
        message2 = json.loads(ctx.output[1], object_pairs_hook=dictionary)
        message3 = json.loads(ctx.output[2], object_pairs_hook=dictionary)
        self.assertDictEqual(
            message1,
            dictionary([
                ("levelname", "INFO"),
                ("msg", "Simple message"),
                (
                    "system",
                    dictionary([(
                        "context",
                        dictionary([("module", "test_json_formatter"),
                                    ("file", "test_json_formatter.py")])
                    )])
                ),
            ])
        )
        self.assertDictEqual(
            message2,
            dictionary([
                ("levelname", "INFO"),
                ("msg", "Composed message: this is a composed message"),
                (
                    "system",
                    dictionary([(
                        "context",
                        dictionary([("module", "test_json_formatter"),
                                    ("file", "test_json_formatter.py")])
                    )])
                ),
            ])
        )
        self.assertDictEqual(
            message3,
            dictionary([
                ("levelname", "INFO"),
                ("msg", "Composed message with extra"),
                (
                    "system",
                    dictionary([(
                        "context",
                        dictionary([("module", "test_json_formatter"),
                                    ("file", "test_json_formatter.py")])
                    )])
                ),
            ])
        )

    def test_embedded_list(self):
        with self.assertLogs('test_formatter', level=logging.DEBUG) as ctx:
            logger = logging.getLogger('test_formatter')
            self._configure_logger(
                logger,
                fmt=dictionary([
                    ("levelname", "levelname"),
                    ("name", "name"),
                    ("message", "message"),
                    (
                        "list", [
                            dictionary([("file", "filename")]),
                            dictionary([("module", "module")]),
                        ]
                    ),
                ])
            )
            logger.info('Composed message %s', 'with list')
        message = json.loads(ctx.output[0], object_pairs_hook=dictionary)
        self.assertDictEqual(
            message,
            dictionary([
                ("levelname", "INFO"),
                ("name", "test_formatter"),
                ("message", "Composed message with list"),
                ('list', [{
                    "file": "test_json_formatter.py"
                }, {
                    "module": "test_json_formatter"
                }]),
            ])
        )

    def test_extra_list(self):
        with self.assertLogs('test_formatter', level=logging.DEBUG) as ctx:
            logger = logging.getLogger('test_formatter')
            self._configure_logger(logger, add_always_extra=True)
            logger.info(
                'Composed message %s',
                'with extra',
                extra={'extra-list': [{
                    "extra-1": "file"
                }, {
                    "extra-2": "module"
                }]}
            )
        message = json.loads(ctx.output[0], object_pairs_hook=dictionary)
        self.assertDictEqual(
            message,
            dictionary([
                ("levelname", "INFO"),
                ("name", "test_formatter"),
                ("message", "Composed message with extra"),
                ('extra-list', [{
                    "extra-1": "file"
                }, {
                    "extra-2": "module"
                }]),
            ])
        )

    def test_exception(self):
        with self.assertLogs('test_formatter', level=logging.DEBUG) as ctx:
            logger = logging.getLogger('test_formatter')
            self._configure_logger(logger)
            try:
                raise ValueError("My value error")
            except ValueError as err:
                logger.exception('Exception message: %s', err)
        message = json.loads(ctx.output[0], object_pairs_hook=dictionary)
        expected_msg = dictionary([
            ("levelname", "ERROR"),
            ("name", "test_formatter"),
            ("message", "Exception message: My value error"),
            (
                "exc_text",
                'Traceback (most recent call last):\n'
                '    raise ValueError("My value error")\n'
                'ValueError: My value error'
            ),
        ])
        self.assertEqual(message.keys(), expected_msg.keys())
        self.assertEqual(message['levelname'], expected_msg['levelname'])
        self.assertEqual(message['name'], expected_msg['name'])
        self.assertEqual(message['message'], expected_msg['message'])
        self.assertEqual(message['exc_text'][:36], expected_msg['exc_text'][:36])
        self.assertEqual(message['exc_text'][-66:], expected_msg['exc_text'][-66:])

    def test_remove_empty(self):
        with self.assertLogs('test_formatter', level=logging.DEBUG) as ctx:
            logger = logging.getLogger('test_formatter')
            self._configure_logger_with_const_attribute(logger, remove_empty=True)
            logger.info('Composed message %s', 'remove empty')
        self.assertDictEqual(
            json.loads(ctx.output[0], object_pairs_hook=dictionary),
            dictionary([
                ("level", "INFO"),
                ("app", "test_application"),
                ("message", "Composed message remove empty"),
            ])
        )

    def test_leave_empty(self):
        with self.assertLogs('test_formatter', level=logging.DEBUG) as ctx:
            logger = logging.getLogger('test_formatter')
            self._configure_logger_with_const_attribute(logger, remove_empty=False)
            logger.info('Composed message %s', 'leave empty')
        self.assertDictEqual(
            json.loads(ctx.output[0], object_pairs_hook=dictionary),
            dictionary([
                ("level", "INFO"),
                ("app", "test_application"),
                ('empty_attr', ''),
                ("message", "Composed message leave empty"),
            ])
        )

    def test_non_serializable_extra(self):

        class TestObject:

            def __str__(self):
                return 'Test Object'

        with self.assertLogs('test_formatter', level=logging.DEBUG) as ctx:
            logger = logging.getLogger('test_formatter')
            self._configure_logger(logger, add_always_extra=True)
            logger.info(
                'Composed message %s',
                'with non serializable extra',
                extra=dictionary([('test-object', TestObject())])
            )
        self.assertDictEqual(
            json.loads(ctx.output[0], object_pairs_hook=dictionary),
            dictionary([
                ("levelname", "INFO"),
                ('name', 'test_formatter'),
                ("message", "Composed message with non serializable extra"),
                ('test-object', 'Test Object'),
            ])
        )



import logging
from collections import OrderedDict
import unittest

from logging_utilities.formatters.extra_formatter import ExtraFormatter


class ExtraFormatterTest(unittest.TestCase):
    maxDiff = None

    @classmethod
    def _configure_logger(
        cls,
        logger,
        fmt=None,
        extra_fmt=None,
        extra_default='',
        extra_pretty_print=False,
        pretty_print_kwargs=None
    ):
        logger.setLevel(logging.DEBUG)

        for handler in logger.handlers:
            formatter = ExtraFormatter(
                fmt,
                extra_default=extra_default,
                extra_fmt=extra_fmt,
                extra_pretty_print=extra_pretty_print,
                pretty_print_kwargs=pretty_print_kwargs
            )
            handler.setFormatter(formatter)

    def test_missing_extra(self):
        with self.assertLogs('test_formatter', level=logging.DEBUG) as ctx:
            logger = logging.getLogger('test_formatter')
            self._configure_logger(logger, fmt="%(message)s", extra_fmt=':%(extra1)s')
            logger.info('Simple message')
            logger.info('Composed message: %s', 'this is a composed message')
            logger.info('Simple message with extra', extra={'extra1': 23})
            logger.info('Composed message %s', 'with extra', extra={'extra1': 23})
        self.assertEqual(ctx.output[0], 'Simple message')
        self.assertEqual(ctx.output[1], 'Composed message: this is a composed message')
        self.assertEqual(ctx.output[2], 'Simple message with extra:23')
        self.assertEqual(ctx.output[3], 'Composed message with extra:23')

    def test_missing_extra_default_none(self):
        with self.assertLogs('test_formatter', level=logging.DEBUG) as ctx:
            logger = logging.getLogger('test_formatter')
            self._configure_logger(
                logger, fmt="%(message)s", extra_fmt=':%(extra1)s:%(extra2)s', extra_default=None
            )
            logger.info('Simple message')
            logger.info('Composed message: %s', 'this is a composed message')
            logger.info('Simple message with extra', extra={'extra1': 23})
            logger.info('Composed message %s', 'with extra', extra={'extra1': 23})
        self.assertEqual(ctx.output[0], 'Simple message')
        self.assertEqual(ctx.output[1], 'Composed message: this is a composed message')
        self.assertEqual(ctx.output[2], 'Simple message with extra:23:None')
        self.assertEqual(ctx.output[3], 'Composed message with extra:23:None')

    def test_extra_format_as_dict(self):
        with self.assertLogs('test_formatter', level=logging.DEBUG) as ctx:
            logger = logging.getLogger('test_formatter')
            self._configure_logger(logger, fmt="%(message)s", extra_fmt=':extra=%s')
            logger.info('Simple message')
            logger.info('Composed message: %s', 'this is a composed message')
            logger.info('Simple message with extra', extra={'extra2': 1})
            logger.info('Composed message %s', 'with extra', extra={'extra2': 1})
        self.assertEqual(ctx.output[0], 'Simple message')
        self.assertEqual(ctx.output[1], 'Composed message: this is a composed message')
        self.assertEqual(ctx.output[2], 'Simple message with extra:extra={\'extra2\': 1}')
        self.assertEqual(ctx.output[3], 'Composed message with extra:extra={\'extra2\': 1}')

    def test_extra_format_as_dict_duplicate_extra(self):
        # Make sure to not replicate extra in the dict that have been added to the standard format
        with self.assertLogs('test_formatter', level=logging.DEBUG) as ctx:
            logger = logging.getLogger('test_formatter')
            self._configure_logger(
                logger, fmt="%(message)s - %(extra1)s", extra_fmt=' - other_extra=%s'
            )
            logger.info('Simple message with extra', extra={'extra1': 1, 'extra2': 2})
            logger.info('Composed message %s', 'with extra', extra={'extra1': 1, 'extra2': 2})
        self.assertEqual(
            ctx.output[0], 'Simple message with extra - 1 - other_extra={\'extra2\': 2}'
        )
        self.assertEqual(
            ctx.output[1], 'Composed message with extra - 1 - other_extra={\'extra2\': 2}'
        )

    def test_extra_format_as_dict_pretty_print(self):
        with self.assertLogs('test_formatter', level=logging.DEBUG) as ctx:
            logger = logging.getLogger('test_formatter')
            self._configure_logger(
                logger, fmt="%(message)s", extra_fmt=':extra=%s', extra_pretty_print=True
            )
            logger.info('Simple message')
            logger.info('Composed message: %s', 'this is a composed message')
            logger.info('Simple message with extra', extra={'extra2': 1})
            logger.info('Composed message %s', 'with extra', extra={'extra2': 1})
            logger.info(
                'Big extra using pretty print',
                extra=OrderedDict([
                    ('extra1', list(map(lambda i: 'test ' * i, range(5)))),
                    ('extra2', {
                        'extra2.1': list(map(lambda i: 'test ' * i, range(5)))
                    }),
                ])
            )
        self.assertEqual(ctx.output[0], 'Simple message')
        self.assertEqual(ctx.output[1], 'Composed message: this is a composed message')
        self.assertEqual(ctx.output[2], 'Simple message with extra:extra={\'extra2\': 1}')
        self.assertEqual(ctx.output[3], 'Composed message with extra:extra={\'extra2\': 1}')
        self.assertEqual(
            ctx.output[4],
            """Big extra using pretty print:extra={'extra1': ['',
            'test ',
            'test test ',
            'test test test ',
            'test test test test '],
 'extra2': {'extra2.1': ['',
                         'test ',
                         'test test ',
                         'test test test ',
                         'test test test test ']}}"""
        )

    def test_extra_format_as_dict_pretty_print_with_args(self):
        with self.assertLogs('test_formatter', level=logging.DEBUG) as ctx:
            logger = logging.getLogger('test_formatter')
            self._configure_logger(
                logger,
                fmt="%(message)s",
                extra_fmt=':extra=%s',
                extra_pretty_print=True,
                pretty_print_kwargs={
                    'indent': 2, 'width': 20
                }
            )
            logger.info('Simple message')
            logger.info('Composed message: %s', 'this is a composed message')
            logger.info('Simple message with extra', extra={'extra2': 1})
            logger.info('Composed message %s', 'with extra', extra={'extra2': 1})
            logger.info(
                'Big extra using pretty print',
                extra=OrderedDict([
                    ('extra1', list(map(lambda i: 'test ' * i, range(5)))),
                    ('extra2', {
                        'extra2.1': list(map(lambda i: 'test ' * i, range(5)))
                    }),
                ])
            )
        self.assertEqual(ctx.output[0], 'Simple message')
        self.assertEqual(ctx.output[1], 'Composed message: this is a composed message')
        self.assertEqual(ctx.output[2], 'Simple message with extra:extra={\'extra2\': 1}')
        self.assertEqual(ctx.output[3], 'Composed message with extra:extra={\'extra2\': 1}')
        # yapf: disable
        self.assertEqual(
            ctx.output[4],
        """Big extra using pretty print:extra={ 'extra1': [ '',
              'test ',
              'test '
              'test ',
              'test '
              'test '
              'test ',
              'test '
              'test '
              'test '
              'test '],
  'extra2': { 'extra2.1': [ '',
                            'test ',
                            'test '
                            'test ',
                            'test '
                            'test '
                            'test ',
                            'test '
                            'test '
                            'test '
                            'test ']}}"""
        )
        # yapf: enable

    def test_extra_format_custom(self):
        with self.assertLogs('test_formatter', level=logging.DEBUG) as ctx:
            logger = logging.getLogger('test_formatter')
            self._configure_logger(
                logger, fmt="%(message)s", extra_fmt=':extra2=%(extra2)s:extra3=%(extra3)s'
            )
            logger.info('Simple message')
            logger.info('Composed message: %s', 'this is a composed message')
            logger.info('Simple message with extra', extra={'extra1': 23, 'extra2': 1})
            logger.info(
                'Composed message %s',
                'with extra',
                extra={
                    'extra1': 23, 'extra2': 1, 'extra3': 'test'
                }
            )
        self.assertEqual(ctx.output[0], 'Simple message')
        self.assertEqual(ctx.output[1], 'Composed message: this is a composed message')
        self.assertEqual(ctx.output[2], 'Simple message with extra:extra2=1:extra3=')
        self.assertEqual(ctx.output[3], 'Composed message with extra:extra2=1:extra3=test')

    def test_extra_format_custom_default_none(self):
        with self.assertLogs('test_formatter', level=logging.DEBUG) as ctx:
            logger = logging.getLogger('test_formatter')
            self._configure_logger(
                logger,
                fmt="%(message)s",
                extra_fmt=':extra2=%(extra2)s:extra3=%(extra3)s',
                extra_default=None
            )
            logger.info('Simple message')
            logger.info('Composed message: %s', 'this is a composed message')
            logger.info('Simple message with extra', extra={'extra1': 23, 'extra2': 1})
            logger.info(
                'Composed message %s',
                'with extra',
                extra={
                    'extra1': 23, 'extra2': 1, 'extra3': 'test'
                }
            )
        self.assertEqual(ctx.output[0], 'Simple message')
        self.assertEqual(ctx.output[1], 'Composed message: this is a composed message')
        self.assertEqual(ctx.output[2], 'Simple message with extra:extra2=1:extra3=None')
        self.assertEqual(ctx.output[3], 'Composed message with extra:extra2=1:extra3=test')

    def test_extra_format_custom_pretty_print(self):
        with self.assertLogs('test_formatter', level=logging.DEBUG) as ctx:
            logger = logging.getLogger('test_formatter')
            self._configure_logger(
                logger,
                fmt="%(message)s",
                extra_fmt=':extra2=%(extra2)s:extra3=%(extra3)s',
                extra_pretty_print=True
            )
            logger.info('Simple message')
            logger.info('Composed message: %s', 'this is a composed message')
            with self.assertRaises(ValueError):
                logger.info('Simple message with extra', extra={'extra1': 23, 'extra2': 1})
                logger.info(
                    'Composed message %s',
                    'with extra',
                    extra={
                        'extra1': 23, 'extra2': 1, 'extra3': 'test'
                    }
                )
        self.assertEqual(ctx.output[0], 'Simple message')
        self.assertEqual(ctx.output[1], 'Composed message: this is a composed message')



import json
import logging
import sys
import unittest
from collections import OrderedDict

from django.conf import settings
from django.test import RequestFactory

from logging_utilities.filters.django_request import JsonDjangoRequest
from logging_utilities.formatters.json_formatter import JsonFormatter

# From python3.7, dict is ordered
if sys.version_info >= (3, 7):
    dictionary = dict
else:
    dictionary = OrderedDict

settings.configure()

logger = logging.getLogger(__name__)


class RecordDjangoAttributesTest(unittest.TestCase):

    def setUp(self):
        # Every test needs access to the request factory.
        self.factory = RequestFactory()

    @classmethod
    def _configure_django_filter(cls, _logger, include_keys=None, exclude_keys=None):
        _logger.setLevel(logging.DEBUG)

        for handler in _logger.handlers:
            django_filter = JsonDjangoRequest(include_keys=include_keys, exclude_keys=exclude_keys)
            handler.addFilter(django_filter)
            formatter = JsonFormatter(
                dictionary([
                    ('level', 'levelname'),
                    ('message', 'message'),
                    ('request', 'request'),
                ]),
                remove_empty=True
            )
            handler.setFormatter(formatter)

    def test_django_include_keys(self):
        # pylint: disable=protected-access
        django_filter = JsonDjangoRequest(
            include_keys=[
                'request.META.METHOD',
                'request.environ',
                'request.environ._include',
            ]
        )
        # True assertions
        self.assertTrue(django_filter._include_key('request', 'request'))
        self.assertTrue(django_filter._include_key('request.META', 'META'))
        self.assertTrue(django_filter._include_key('request.META.METHOD', 'METHOD'))
        self.assertTrue(django_filter._include_key('request.environ', 'environ'))
        self.assertTrue(django_filter._include_key('request.environ.CONTENT_TYPE', 'CONTENT_TYPE'))
        self.assertTrue(django_filter._include_key('request.environ._include', '_include'))
        # False assertions
        self.assertFalse(django_filter._include_key('test', 'test'))
        self.assertFalse(django_filter._include_key('request.path', 'path'))
        self.assertFalse(django_filter._include_key('request.path.full', 'full'))
        self.assertFalse(django_filter._include_key('request.META.CONTENT_TYPE', 'CONTENT_TYPE'))
        self.assertFalse(django_filter._include_key('request.META.extend.TYPE', 'TYPE'))
        self.assertFalse(django_filter._include_key('request.environ._type', '_type'))

    def test_django_exclude_keys(self):
        # pylint: disable=protected-access
        django_filter = JsonDjangoRequest(exclude_keys=['request.META.METHOD', 'request.environ'])
        # True assertions
        self.assertTrue(django_filter._exclude_key('request.META.METHOD', 'METHOD'))
        self.assertTrue(django_filter._exclude_key('request.environ', 'environ'))
        self.assertTrue(django_filter._exclude_key('request.environ.CONTENT_TYPE', 'CONTENT_TYPE'))
        self.assertTrue(django_filter._exclude_key('request.environ._include', '_include'))
        self.assertTrue(django_filter._exclude_key('request.environ._type', '_type'))
        # False assertions
        self.assertFalse(django_filter._exclude_key('test', 'test'))
        self.assertFalse(django_filter._exclude_key('request.path', 'path'))
        self.assertFalse(django_filter._exclude_key('request.path.full', 'full'))
        self.assertFalse(django_filter._exclude_key('request.META.CONTENT_TYPE', 'CONTENT_TYPE'))
        self.assertFalse(django_filter._exclude_key('request.META.extend.TYPE', 'TYPE'))
        self.assertFalse(django_filter._exclude_key('request', 'request'))
        self.assertFalse(django_filter._exclude_key('request.META', 'META'))

    def test_django_request_jsonify(self):
        request = self.factory.get('/my_path?test=true&test_2=false')
        with self.assertLogs('test_formatter', level=logging.DEBUG) as ctx:
            test_logger = logging.getLogger('test_formatter')
            self._configure_django_filter(
                test_logger,
                include_keys=[
                    'request.META.REQUEST_METHOD', 'request.META.SERVER_NAME', 'request.environ'
                ],
                exclude_keys=['request.META.SERVER_NAME', 'request.environ.wsgi']
            )
            test_logger.info('Simple message', extra={'request': request})
            test_logger.info(
                'Composed message: %s', 'this is a composed message', extra={'request': request}
            )
        message1 = json.loads(ctx.output[0], object_pairs_hook=dictionary)
        message2 = json.loads(ctx.output[1], object_pairs_hook=dictionary)
        logger.debug('message1=%s', message1)
        logger.debug('message2=%s', message2)
        self.assertDictEqual(
            message1,
            dictionary([
                ("level", "INFO"),
                ("message", "Simple message"),
                (
                    "request",
                    dictionary([("META", dictionary([("REQUEST_METHOD", "GET")])),
                                (
                                    "environ",
                                    dictionary([
                                        ("HTTP_COOKIE", ""),
                                        ("PATH_INFO", "/my_path"),
                                        ("QUERY_STRING", "test=true&test_2=false"),
                                        ("REMOTE_ADDR", "127.0.0.1"),
                                        ("REQUEST_METHOD", "GET"),
                                        ("SCRIPT_NAME", ""),
                                        ("SERVER_NAME", "testserver"),
                                        ("SERVER_PORT", "80"),
                                        ("SERVER_PROTOCOL", "HTTP/1.1"),
                                    ])
                                )])
                ),
            ]),
            msg="First message differ"
        )
        self.assertDictEqual(
            message2,
            dictionary([("level", "INFO"),
                        ("message", "Composed message: this is a composed message"),
                        (
                            "request",
                            dictionary([("META", dictionary([("REQUEST_METHOD", "GET")])),
                                        (
                                            "environ",
                                            dictionary([
                                                ("HTTP_COOKIE", ""),
                                                ("PATH_INFO", "/my_path"),
                                                ("QUERY_STRING", "test=true&test_2=false"),
                                                ("REMOTE_ADDR", "127.0.0.1"),
                                                ("REQUEST_METHOD", "GET"),
                                                ("SCRIPT_NAME", ""),
                                                ("SERVER_NAME", "testserver"),
                                                ("SERVER_PORT", "80"),
                                                ("SERVER_PROTOCOL", "HTTP/1.1"),
                                            ])
                                        )])
                        )]),
            msg="Second message differ"
        )



import json
import logging
import logging.config
import sys
import warnings
from collections import OrderedDict

from logging_utilities.formatters import RECORD_DFT_ATTR

if sys.version_info < (3, 0):
    raise ImportError('Only python 3 is supported')

# From python3.7, dict is ordered. Ordered dict are preferred in order to keep the json output
# in the same order as its definition
if sys.version_info >= (3, 7):
    dictionary = dict
else:
    dictionary = OrderedDict

DEFAULT_FORMAT = dictionary([('levelname', 'levelname'), ('name', 'name'), ('message', 'message')])


class JsonFormatter(logging.Formatter):
    """Logging JSON Formatter

    This formatter transform the log message into a JSON object.
    """

    def __init__(
        self,
        fmt=None,
        datefmt=None,
        style='%',
        add_always_extra=False,
        filter_attributes=None,
        remove_empty=False,
        **kwargs
    ):
        """JSON Formatter constructor

        Args:
            fmt: (dict)
                Format for the json output. It can be a string representing a json object or
                a dictionary. The object defines the JSON output:
                    KEY   := key to use in the json output
                    VALUE := value to use in the json output, this can be either a static string,
                             a record attribute name or a format string
            datefmt: (string)
                Format string for the date format
            style: (string)
                Use a style parameter of '%', '{' or '$' to specify that you want to use one of
                %-formatting, :meth:`str.format` (``{}``) formatting or :class:`string.Template`
                formatting in your format string.
            add_always_extra: (bool)
                If true the extra dictionary passed to the log function will be always
                added to the json output. Otherwise it is only added if the keys are in the
                fmt.
            filter_attributes: (list)
                List of record attributes added by a filter or log addapter.
                This list is used to differentiate record attributes from log extra
                attributes.
            remove_empty: (bool)
                    If true the empty list, object or string in the output message are removed
            kwargs:
                Additional parameters passed to json.dumps()

        Raises:
            TypeError:  When the fmt parameter is in a wrong type
            json.decoder.JSONDecodeError: When fmt is a string that don't describe a json object
        """
        super().__init__(fmt='', datefmt=datefmt, style=style)

        self.json_fmt = self._parse_fmt(fmt)
        self.add_always_extra = add_always_extra
        self.filter_attributes = filter_attributes
        self.remove_empty = remove_empty

        # support for `json.dumps` parameters
        self.kwargs = kwargs

    @classmethod
    def _parse_fmt(cls, fmt):
        if isinstance(fmt, str):
            return json.loads(fmt, object_pairs_hook=dictionary)
        if isinstance(fmt, (dictionary, OrderedDict)):
            return fmt
        if isinstance(fmt, dict):
            warnings.warn(
                "Current python version is lower than 3.7.0, the key's order of dict may be "
                "different than the definition, please use `OrderedDict` instead.",
                UserWarning
            )
            return dictionary((k, fmt[k]) for k in sorted(fmt.keys()))
        if fmt is None:
            return DEFAULT_FORMAT

        raise TypeError(
            '`{}` type is not supported, `fmt` must be json `str`, `OrderedDict` or `dict` type.'.
            format(type(fmt))
        )

    @classmethod
    def _add_extra_to_message(cls, extra, message):
        for key, value in extra.items():
            message[key] = value

    def _get_extra_attrs(self, record):

        def is_extra_attribute(attr):
            if attr not in RECORD_DFT_ATTR and (
                self.filter_attributes is None or attr not in self.filter_attributes
            ):
                return True
            return False

        extras = {key: record.__dict__[key] for key in record.__dict__ if is_extra_attribute(key)}
        if sys.version_info >= (3, 7):
            return extras
        return dictionary((key, extras[key]) for key in sorted(extras.keys()))

    def _add_list_to_message(self, record, lst, message):
        for value in lst:
            if isinstance(value, (dict, OrderedDict)):
                intermediate_msg = dictionary()
                self._add_object_to_message(record, value, intermediate_msg)
                if not self.remove_empty or len(intermediate_msg) > 0:
                    message.append(intermediate_msg)
            elif isinstance(value, list):
                intermediate_msg = list()
                self._add_list_to_message(record, value, intermediate_msg)
                if not self.remove_empty or len(intermediate_msg) > 0:
                    message.append(intermediate_msg)
            elif value == 'exc_info':
                # exc_info might be true or contain exception info. We add it here only a boolean to
                # know if the message has exception info or not. The info itself will be in
                # 'exc_text' and is always appended to the json output when not available
                message.append(bool(record.exc_info))
            elif value in record.__dict__:
                intermediate_msg = getattr(record, value, None)
                if not self.remove_empty or intermediate_msg is not None:
                    message.append(intermediate_msg)
            elif isinstance(value, str):
                # When the value is a string it can contain formatting strings (e.g. "%(asctime)s")
                # therefore try to format it.
                self._style._fmt = value  # pylint: disable=protected-access
                intermediate_msg = super().formatMessage(record)
                if not self.remove_empty or intermediate_msg != '':
                    message.append(intermediate_msg)

    def _add_object_to_message(self, record, obj, message):
        for key, value in obj.items():
            if isinstance(value, (dict, OrderedDict)):
                message[key] = dictionary()
                self._add_object_to_message(record, value, message[key])
                if self.remove_empty and len(message[key]) == 0:
                    del message[key]
            elif isinstance(value, list):
                message[key] = list()
                self._add_list_to_message(record, value, message[key])
                if self.remove_empty and len(message[key]) == 0:
                    del message[key]
            elif value == 'exc_info':
                # exc_info might be true or contain exception info. We add it here only a boolean to
                # know if the message has exception info or not. The info itself will be in
                # 'exc_text' and is always appended to the json output when not available
                message[key] = bool(record.exc_info)
            elif value in record.__dict__:
                message[key] = getattr(record, value, '')
                if self.remove_empty and message[key] == '':
                    del message[key]
            elif isinstance(value, str):
                # When the value is a string it can contain formatting strings (e.g. "%(asctime)s")
                # therefore try to format it.
                self._style._fmt = value  # pylint: disable=protected-access
                message[key] = super().formatMessage(record)
                if self.remove_empty and message[key] == '':
                    del message[key]

    def formatMessage(self, record):
        message = dictionary()
        self._add_object_to_message(record, self.json_fmt, message)
        return message

    def format(self, record):
        record.message = record.getMessage()
        if self.usesTime():
            record.asctime = self.formatTime(record, self.datefmt)

        if self.add_always_extra:
            extra = self._get_extra_attrs(record)

        message = self.formatMessage(record)

        if self.add_always_extra:
            self._add_extra_to_message(extra, message)

        if record.exc_info:
            # Cache the traceback text to avoid converting it multiple times
            # (it's constant anyway)
            if not record.exc_text:
                record.exc_text = self.formatException(record.exc_info)
        if record.exc_text:
            message['exc_text'] = record.exc_text

        if record.stack_info:
            message['stack_info'] = self.formatStack(record.stack_info)

        # When adding all extras, to avoid crash when a log message adds an extra with a non
        # serializable object, we add a default serializer.
        default = self.kwargs.pop('default', None)
        if self.add_always_extra and default is None:
            default = str

        return json.dumps(message, default=default, **self.kwargs)


def basic_config(**kwargs):
    """
    Do basic configuration for the logging system.

    This function does nothing if the root logger already has handlers
    configured. It is a convenience method intended for use by simple scripts
    to do one-shot configuration of the logging package.

    The default behavior is to create a StreamHandler which writes to
    sys.stderr, set a formatter using the BASIC_FORMAT format string, and
    add the handler to the root logger.

    A number of optional keyword arguments may be specified, which can alter
    the default behavior.

    Args:
        filename:
            Specifies that a FileHandler be created, using the specified filename,
            rather than a StreamHandler.
        filemode:
            Specifies the mode to open the file, if filename is specified
            (if filemode is unspecified, it defaults to 'a').
        format:
            Use the specified format string for the handler.
        datefmt:
            Use the specified date/time format.
        style:
            If a format string is specified, use this to specify the
            type of format string (possible values '%', '{', '$', for
            %-formatting, :meth:`str.format` and :class:`string.Template`
            - defaults to '%').
        level:
            Set the root logger level to the specified level.
        stream:
            Use the specified stream to initialize the StreamHandler. Note
            that this argument is incompatible with 'filename' - if both
            are present, 'stream' is ignored.
        handlers:
            If specified, this should be an iterable of already created
            handlers, which will be added to the root handler. Any handler
            in the list which does not have a formatter assigned will be
            assigned the formatter created in this function.

    Note that you could specify a stream created using open(filename, mode)
    rather than passing the filename and mode in. However, it should be
    remembered that StreamHandler does not close its stream (since it may be
    using sys.stdout or sys.stderr), whereas FileHandler closes its stream
    when the handler is closed.
    """
    # Add thread safety in case someone mistakenly calls
    # basic_config() from multiple threads
    # pylint: disable=protected-access
    logging._acquireLock()
    try:
        if len(logging.root.handlers) > 0:
            return

        handlers = kwargs.pop("handlers", None)
        if handlers is None:
            if "stream" in kwargs and "filename" in kwargs:
                raise ValueError("'stream' and 'filename' should not be specified together")
        else:
            if "stream" in kwargs or "filename" in kwargs:
                raise ValueError(
                    "'stream' or 'filename' should not be specified together with 'handlers'"
                )
        if handlers is None:
            filename = kwargs.pop("filename", None)
            mode = kwargs.pop("filemode", 'a')
            if filename:
                handler = logging.FileHandler(filename, mode)
            else:
                stream = kwargs.pop("stream", None)
                handler = logging.StreamHandler(stream)
            handlers = [handler]
        dfs = kwargs.pop("datefmt", None)
        style = kwargs.pop("style", '%')
        if style not in logging._STYLES:
            raise ValueError('Style must be one of: %s' % ','.join(logging._STYLES.keys()))
        fmt = kwargs.pop("format", DEFAULT_FORMAT)
        formatter = JsonFormatter(fmt, dfs, style)
        for handler in handlers:
            if handler.formatter is None:
                handler.setFormatter(formatter)
            logging.root.addHandler(handler)
        level = kwargs.pop("level", None)
        if level is not None:
            logging.root.setLevel(level)
        if kwargs:
            keys = ', '.join(kwargs.keys())
            raise ValueError('Unrecognized argument(s): %s' % keys)
    finally:
        logging._releaseLock()



import logging
import re
import sys
import warnings
from collections import OrderedDict

from django.http import HttpRequest

# From python3.7, dict is ordered. Ordered dict are preferred in order to keep the json output
# in the same order as its definition
if sys.version_info >= (3, 7):
    dictionary = dict
else:
    dictionary = OrderedDict


def _pattern(text):
    return r'^{}\b'.format(text.replace('.', '[.]'))


class JsonDjangoRequest(logging.Filter):
    """Convert Django request to a json object

    The django framework adds sometimes a request to the logs extra (HttpRequest or socket object).
    This filter recursively converts this request object to a python dictionary that can be dumped
    into a json string.
    This is useful for example if you want to use this request with the JSON formatter.

    Additionally the attributes of the request that needs to be jsonify can be configured using the
    `include_keys` and/or `exclude_keys` parameters.
    """

    def __init__(self, include_keys=None, exclude_keys=None):
        """Initialize the filter

        Args:
            include_keys: (list | None)
                All request attributes that match any of the dotted keys of the list will be
                jsonify in the record.request. When None then all attributes are added
                (default behavior).
            exclude_keys: (list | None)
                All request attributes that match any of the dotted keys of the list will not be
                added to the jsonify of the record.request. NOTE this has precedence to include_keys
                which means that if a key is in both list, then it is not added.
        """
        self.include_keys = include_keys
        self.exclude_keys = exclude_keys
        super().__init__()

    def filter(self, record):
        if not hasattr(record, 'request'):
            return True

        self._jsonify_request(record)

        return True

    def _jsonify_request(self, record):
        if isinstance(record.request, HttpRequest) and hasattr(record.request, '__dict__'):
            request = self._jsonify_dict('request', record.request.__dict__)
            if self._add_key('request.headers', 'headers'):
                # HttpRequest has a special headers property that is cached and is not always in
                # record.request.__dict__
                request['headers'] = self._jsonify_dict('request.headers', record.request.headers)
        else:
            # Django sets also in some log message extra={'request': socket.socket()} in this case
            # we simply stringify it
            request = str(record.request)
        setattr(record, 'request', request)

    def _jsonify_dict(self, prefix, dct):
        json_obj = dictionary()
        if sys.version_info < (3, 7):
            dct = OrderedDict(sorted(dct.items(), key=lambda t: t[0]))
        for key, value in dct.items():
            dotted_key = '{}.{}'.format(prefix, key)
            if not self._add_key(dotted_key, key):
                continue
            if hasattr(value, '__dict__'):
                json_obj[key] = self._jsonify_dict(dotted_key, value.__dict__)
            elif isinstance(value, dict):
                json_obj[key] = self._jsonify_dict(dotted_key, value)
            elif isinstance(value, (str, int, float, type(None), bool, tuple, list)):
                json_obj[key] = value
            elif isinstance(value, (bytes)):
                json_obj[key] = str(value)
            else:
                warnings.warn(
                    "Cannot jsonify key {} with value {}: unsupported type={}".format(
                        dotted_key, value, type(value)
                    )
                )
                json_obj[key] = str(value)
        return json_obj

    def _add_key(self, dotted_key, key):
        return self._include_key(dotted_key, key) and not self._exclude_key(dotted_key, key)

    def _include_key(self, dotted_key, key):
        if self.include_keys is None:
            # if no include_keys is configured add all keys except for private
            if key.startswith('_'):
                return False
            return True

        def match(item):
            return (
                re.match(_pattern(dotted_key), item) or
                (re.match(_pattern(item), dotted_key) and not key.startswith('_'))
            )

        return any(map(match, self.include_keys))

    def _exclude_key(self, dotted_key, key):
        if self.exclude_keys is None:
            # if no exclude_keys is configured only exclude private key
            if key.startswith('_'):
                return True
            return False

        def match(item):
            return re.match(_pattern(item), dotted_key)

        return any(map(match, self.exclude_keys))



import logging
from datetime import datetime, timezone


class ConstAttribute(logging.Filter):
    '''Logging constant record attribute

    This filter add a constant attribute to the log record.
    '''

    def __init__(self, **kwargs):
        '''Initialize filter

        Args:
            kwargs:
                All keyword arguments are added as LogRecord attributes
        '''
        self.kwargs = kwargs
        super().__init__()

    def filter(self, record):
        for key, value in self.kwargs.items():
            setattr(record, key, value)
        return True


class LevelFilter(logging.Filter):
    '''Logging level filter

    This filter can be used on a handler to filter logger message based on their level.

    For example if you have two DEBUG loggers; A and B, with both two handlers; console and file.
    On the file handler you want all message of A but only WARNING message of B. You can then use
    the following configuration:

    root:
        handlers:
            - "console"
            - "file"
        level: "DEBUG"
        propagate: "True"

    filters:
        BFilter:
            class: logging_utilities.filters.LevelFilter
            level: "WARNING"
            logger: 'B'

    loggers:
        A:
            level: "DEBUG"
        B:
            level: "DEBUG"

    handlers:
        console:
            class: "logging.StreamHandler"

        file:
            class: "logging.handlers.RotatingFileHandler"
            filters:
                - "BFilter"
    '''

    def __init__(self, level='DEBUG', logger=''):
        '''Initialize the filter

        Args:
            level: (str|int)
                Level to filter, all message with a lower level will be filtered
            logger: (str)
                Logger name on which to apply the level filtering, if empty then the filtering is
                applied to all loggers

        Raises:
            ValueError: when an invalid level is given
        '''
        if not isinstance(level, (str, int)):
            raise ValueError('Unsupported level type: must be int or string')
        self.level = level
        if isinstance(self.level, str):
            # translate level to int
            self.level = logging.getLevelName(self.level)
            if not isinstance(self.level, int):
                raise ValueError('Unsupported level string')
        elif isinstance(self.level, int) \
            and logging.getLevelName(self.level) == "Level %d" % (self.level):
            raise ValueError('Undefined level integer')

        self.logger = logger
        super().__init__()

    def filter(self, record):
        if self.logger == '' or record.name.startswith(self.logger):
            if record.levelno < self.level:
                return False
        return True


class TimeAttribute(logging.Filter):
    '''Logging time record attribute

    This filter can be used on a handler to add iso 8601 time attribute to the record.
    '''

    def __init__(self, isotime=True, utc_isotime=False):
        '''Initialize the filter

        Args:
            isotime: (bool)
                Add local time in `YYYY-MM-DDThh:mm:ss±hh:mm` format as `isotime`
                attribute to LogRecord
            utc_isotime: (bool)
                Add utc time in `YYYY-MM-DDThh:mm:ss±hh:mm` format as `utc_isotime`
                attribute to LogRecord
        '''
        self.isotime = isotime
        self.utc_isotime = utc_isotime
        super().__init__()

    def filter(self, record):
        if self.isotime:
            record.isotime = datetime.fromtimestamp(record.created).astimezone().isoformat()
        if self.utc_isotime:
            record.utc_isotime = datetime.fromtimestamp(record.created, tz=timezone.utc) \
                .isoformat().replace('+00:00', 'Z')
        return True



import logging
import unittest
from logging import Formatter

from logging_utilities.filters import ConstAttribute
from logging_utilities.filters.flask_attribute import FlaskRequestAttribute


class RecordAttributesTest(unittest.TestCase):

    @classmethod
    def _configure_const_attribute(cls, logger):
        logger.setLevel(logging.DEBUG)

        for handler in logger.handlers:
            const_attribute = ConstAttribute(application="test_application")
            handler.addFilter(const_attribute)
            handler.setFormatter(Formatter("%(levelname)s:%(application)s:%(message)s"))

    @classmethod
    def _configure_flask_attribute(cls, logger):
        logger.setLevel(logging.DEBUG)

        for handler in logger.handlers:
            const_attribute = FlaskRequestAttribute(attributes=['url', 'method', 'headers', 'json'])
            handler.addFilter(const_attribute)
            handler.setFormatter(Formatter("%(levelname)s:%(message)s:%(flask_request_url)s"))

    def test_const_attribute(self):
        with self.assertLogs('test_formatter', level=logging.DEBUG) as ctx:
            logger = logging.getLogger('test_formatter')
            self._configure_const_attribute(logger)
            logger.info('Simple message')
            logger.info('Composed message: %s', 'this is a composed message')
            logger.info('Composed message %s', 'with extra', extra={'extra1': 23})
        self.assertEqual(
            ctx.output,
            [
                'INFO:test_application:Simple message',
                'INFO:test_application:Composed message: this is a composed message',
                'INFO:test_application:Composed message with extra'
            ]
        )

    def test_empty_flask_attribute(self):
        with self.assertLogs('test_formatter', level=logging.DEBUG) as ctx:
            logger = logging.getLogger('test_formatter')
            self._configure_flask_attribute(logger)
            logger.info('Simple message')
            logger.info('Composed message: %s', 'this is a composed message')
            logger.info('Composed message %s', 'with extra', extra={'extra1': 23})
        self.assertEqual(
            ctx.output,
            [
                'INFO:Simple message:',
                'INFO:Composed message: this is a composed message:',
                'INFO:Composed message with extra:'
            ]
        )



import logging

from flask import has_request_context
from flask import request


class FlaskRequestAttribute(logging.Filter):
    """Logging Flask attributes record

    This filter adds Flask request context attributes to the log record.

    Flask request attributes are added as record attributes with the 'flask_request_' prefix.
    """

    def __init__(self, attributes=None):
        """Initialize the filter

        Args:
            attributes: (list)
                Flask request attribute names list to add to the log record
        """
        super().__init__()
        self.attributes = attributes if attributes else list()

    def filter(self, record):
        for attribute in self.attributes:
            rec_attribute = 'flask_request_' + attribute
            value = ''
            if has_request_context():
                value = getattr(request, attribute, '')
            if value is None or isinstance(value, (str, int, float, dict)):
                setattr(record, rec_attribute, value)
            elif attribute == 'headers':
                setattr(record, rec_attribute, dict(value.items()))
            else:
                ValueError('Attribute %s=%s unsupported type' % (attribute, value))
        return True



#!/usr/bin/env python

from setuptools import setup, find_packages

# Get description from README.md
LONG_DESCRIPTION = ''
with open('README.md', encoding='utf-8') as rd:
    LONG_DESCRIPTION = rd.read()

# Here we cannot import the version but need to read it otherwise we might have an ImportError
# during execution of the setup.py if the package contains other libaries.
VERSION_LINE = list(
    filter(lambda l: l.startswith('VERSION'), open('./logging_utilities/__init__.py'))
)[0]


def get_version(version_line):
    # pylint: disable=eval-used
    version_tuple = eval(version_line.split('=')[-1])
    return ".".join(map(str, version_tuple))


setup(
    name='logging-utilities',
    version=get_version(VERSION_LINE),
    description=('Python logging utilities'),
    long_description=LONG_DESCRIPTION,
    long_description_content_type="text/markdown",
    platforms=["all"],
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3 :: Only',
        'Topic :: Utilities',
        'Topic :: System :: Logging'
    ],
    python_requires='>=3.0',
    author='ltshb',
    author_email='brice.schaffner@swisstopo.ch',
    url='https://github.com/geoadmin/lib-py-logging-utilities',
    license='BSD 3-Clause License',
    packages=find_packages()
)



import logging
import logging.config
import re
import sys
from pprint import pformat

from logging_utilities.formatters import RECORD_DFT_ATTR

if sys.version_info < (3, 2):
    raise ImportError('Only python 3.2 and above are supported')

# parse the format to retrieve all key e.g. "%(message)s %(module)s" => ['message', 'module']
KEYS_PATTERN = r'%\((\w+)\)[#0\- \+]?(?:\d+|\*)?(?:\.\d+|\.\*)?\d*[diouxXeEfFgGcrsa]'


class ExtraFormatter(logging.Formatter):
    """Logging Extra Formatter

    This formatter enhance the python standard formatter to allow working with the log `extra`.
    The python standard formatter will raise a ValueError() when adding extra keyword in the format
    when this keyword is then missing from log record. This means that if you want to display a log
    extra, you have to make sure that every log message contains this extra.

    This formatter allow you to provide an `extra_fmt` parameter that will add record extra to the
    log message when available. You can either add the entire extra dictionary: `extra_fmt='%s'` or
    only some extras: `extra_fmt='%(extra1)s:%(extra2)s'`. In the latest case, when a key is missing
    in extra, the value is replaced by `extra_default`.
    When using the whole `extra` dictionary, you can use `extra_pretty_print` to improve the
    formatting, note that in this case the log might be on multiline (this use pprint.pformat).
    """

    def __init__(
        self,
        fmt=None,
        datefmt=None,
        style='%',
        validate=True,
        extra_fmt=None,
        extra_default='',
        extra_pretty_print=False,
        pretty_print_kwargs=None
    ):
        '''
        Initialize the formatter with specified format strings.

        Initialize the formatter either with the specified format string, or a default as described
        in logging.Formatter.

        Args:
            extra_fmt: string
                String format (old percent style only) for log extras. This can be used for instance
                to automatically add all extras, e.g: `extra_fmt='extras=%s'` or to add only some
                extra: `extra_fmt='%(extra1)s:%(extra2)s'`
            extra_default: any
                Default value to use for missing extra in record
            extra_pretty_print: boolean
                Set to true to use pprint.pformat on the extra dictionary
        '''
        super().__init__(fmt=fmt, datefmt=datefmt, style=style)
        self._fmt_keys = re.findall(KEYS_PATTERN, fmt)
        self.extra_fmt = extra_fmt
        self._extras_keys = re.findall(KEYS_PATTERN, self.extra_fmt if self.extra_fmt else '')
        self._default = extra_default
        self._extra_pretty_print = extra_pretty_print
        self._pretty_print_kwargs = pretty_print_kwargs if pretty_print_kwargs is not None else {}

    def formatMessage(self, record):
        message = self._style.format(record)
        if self.extra_fmt:
            extra_keys = set(record.__dict__.keys()) - RECORD_DFT_ATTR - set(self._fmt_keys)
            extras = {key: getattr(record, key) for key in extra_keys}
            if extras:
                missing_keys = set(self._extras_keys) - set(extras.keys())
                extras.update({key: self._default for key in missing_keys})
                if self._extra_pretty_print:
                    try:
                        message = '%s%s' % (
                            message, self.extra_fmt % pformat(extras, **self._pretty_print_kwargs)
                        )
                    except TypeError as err:
                        if err.args[0] == 'format requires a mapping':
                            raise ValueError(
                                'Cannot use extra_pretty_print with named placeholder'
                            ) from err
                        raise err
                else:
                    message = '%s%s' % (message, self.extra_fmt % extras)
        return message
